/*
 * This tests a flow for the cloudron autoupdating itself in sfo region.
 * It checks if an existing installed app retains it's data after
 * an update. It then check if the cloudron can be auto-updated to
 * the next (unreleased) version.
 */

'use strict';

var AppStore = require('../appstore.js'),
    assert = require('assert'),
    Cloudron = require('../cloudron.js'),
    common = require('../common.js'),
    mailer = require('../mailer.js'),
    request = require('superagent-sync'),
    semver = require('semver'),
    sleep = require('../shell.js').sleep;

require('colors');

process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';

var BOX_VERSION = process.env.BOX_VERSION;

describe('Cloudron update testing', function () {
    this.timeout(0);

    var appStore = new AppStore('https://api.staging.cloudron.io');

    var owner = common.getOwner();
    var admin = common.getAdmin();
    var res, fromVersion, toVersion, cloudron, appId, box, nextVersion;

    it('can query versions', function () {
        res = request.get('https://s3.amazonaws.com/staging-cloudron-releases/versions.json').end();
        common.verifyResponse2xx(res);
        var boxVersions = Object.keys(common.stripUnreachable(res.body)).sort(semver.rcompare);
        fromVersion = boxVersions[2]; // we released a new version in before.js
        toVersion = boxVersions[1];
        assert.strictEqual(toVersion, BOX_VERSION);
        nextVersion = boxVersions[0];

        console.log('Will test update from %s to %s and then %s', fromVersion.yellow, toVersion.yellow, nextVersion.yellow);
    });

    it('can login to the store', function () {
        var accessToken = appStore.getAccessToken(owner);
        appStore.setCredentials(owner.password, accessToken);

        var adminAccessToken = appStore.getAccessToken(admin);
        appStore.setAdminCredentials(admin.password, adminAccessToken);
    });

    it('can create a cloudron', function () {
        box = appStore.createCloudron({
            domain: common.cloudronDomain(__filename),
            region: 'sfo1',
            size: '1gb',
            version: fromVersion
        });
        box = appStore.waitForCloudron(box.id);
        cloudron = new Cloudron(box);
    });

    it('can activate the box', function () {
        cloudron.activate(owner);
    });

    it('send mail to cloudron user', function (done) {
        mailer.sendMailToCloudronUser(owner.username + '@' + box.domain, done);
    });

    it('can login to the box', function () {
        var token = cloudron.getOauthToken(owner);
        cloudron.setCredentials(owner.password, token);
    });

    var location = 'test' + (Math.random() * 10000).toFixed();
    it('can install app', function () {
        var manifest = appStore.getManifest(common.TESTAPP_ID, common.PREV_TESTAPP_VERSION);
        appId = cloudron.installApp(location, common.TESTAPP_ID + '@' + common.PREV_TESTAPP_VERSION);
    });

    it('can populate the addons', function () {
        cloudron.populateAddons(cloudron.appFqdn(location));
    });

    it('can check the addons', function () {
        cloudron.checkAddons(cloudron.appFqdn(location), owner);
    });

    it('can update the box', function () {
        cloudron.checkForUpdates();
        cloudron.update(toVersion);
    });

    xit('can login to the box', function () {
        var token = cloudron.getOauthToken(owner);
        cloudron.setCredentials(owner.password, token);
    });

    it('runs the app', function () {
        cloudron.waitForApp(appId);
    });

    it('can check the addons', function () {
        cloudron.checkAddons(cloudron.appFqdn(location), owner);
    });

    it('can set the autoupdate pattern', function () {
        cloudron.setAutoupdatePattern('00 */1 * * * *'); // every 1 minute (but updateChecker runs only every 10 mins :/)
    });

    it('can update to new version', function () {
        console.log('Wait for cloudron to get the update');
        cloudron.checkForUpdates();
        sleep(60 * 10); // give it 10 mins to autoupdate the box and the app
        cloudron.waitForUpdate(nextVersion);
    });

    it('auto updates the app with new version', function () {
        cloudron.checkForUpdates();
        sleep(60 * 10); // give it 10 mins to autoupdate the box and the app
        cloudron.waitForApp(appId, common.TESTAPP_VERSION);
    });

    it('can check the addons', function () {
        cloudron.checkAddons(cloudron.appFqdn(location), owner);
    });

    it('can uninstall app', function () {
        cloudron.uninstallApp(appId);
    });

    it('can check mail', function (done) {
        cloudron.checkMail(owner, done);
    });

    it('can delete the cloudron', function () {
        appStore.deleteCloudron(box);
    });

});

